@@ -25,6 +25,7 @@
 #include <string.h>
 #include "svn_pools.h"
 #include "svn_error.h"
+#include "svn_dirent_uri.h"
 #include "svn_path.h"
 
 #include "wc.h"
@@ -39,10 +40,43 @@
 
 /*** Code. ***/
 
+/* Copy all properties of SRC_PATH to DST_PATH. */
+static svn_error_t *
+copy_props(const char *src_path,
+           const char *dst_path,
+           svn_wc_adm_access_t *src_access,
+           svn_wc_adm_access_t *dst_access,
+           apr_pool_t *pool)
+{
+  apr_hash_t *props;
+  apr_hash_index_t *hi;
+
+  SVN_ERR(svn_wc_prop_list(&props, src_path, src_access, pool));
+  for (hi = apr_hash_first(pool, props); hi; hi = apr_hash_next(hi))
+    {
+      const char *propname;
+      svn_string_t *propval;
+      const void *key;
+      void *val;
+
+      apr_hash_this(hi, &key, NULL, &val);
+      propname = key;
+      propval = val;
+
+      SVN_ERR(svn_wc_prop_set3(propname, propval,
+                               dst_path, dst_access,
+                               FALSE /* skip_checks */,
+                               NULL, NULL, pool));
+    }
+
+  return SVN_NO_ERROR;
+}
+
+
 /* Helper function for svn_wc_copy2() which handles WC->WC copying of
    files which are scheduled for addition or unversioned.
 
-   Copy file SRC_PATH to DST_BASENAME in DST_PARENT_ACCESS.
+   Copy file SRC_PATH in SRC_ACCESS to DST_BASENAME in DST_PARENT_ACCESS.
 
    DST_PARENT_ACCESS is a 0 depth locked access for a versioned directory
    in the same WC as SRC_PATH.
@@ -58,6 +92,7 @@
 static svn_error_t *
 copy_added_file_administratively(const char *src_path,
                                  svn_boolean_t src_is_added,
+                                 svn_wc_adm_access_t *src_access,
                                  svn_wc_adm_access_t *dst_parent_access,
                                  const char *dst_basename,
                                  svn_cancel_func_t cancel_func,
@@ -75,10 +110,14 @@ copy_added_file_administratively(const char *src_path,
 
   if (src_is_added)
     {
-      SVN_ERR(svn_wc_add2(dst_path, dst_parent_access, NULL,
-                          SVN_INVALID_REVNUM, cancel_func,
+      SVN_ERR(svn_wc_add3(dst_path, dst_parent_access, svn_depth_infinity,
+                          NULL, SVN_INVALID_REVNUM, cancel_func,
                           cancel_baton, notify_func,
                           notify_baton, pool));
+
+      SVN_ERR(copy_props(src_path, dst_path,
+                         src_access, dst_parent_access,
+                         pool));
     }
 
   return SVN_NO_ERROR;
@@ -150,10 +189,15 @@ copy_added_dir_administratively(const char *src_path,
 
       /* Add the directory, adding locking access for dst_path
          to dst_parent_access at the same time. */
-      SVN_ERR(svn_wc_add2(dst_path, dst_parent_access, NULL,
+      SVN_ERR(svn_wc_add3(dst_path, dst_parent_access, svn_depth_infinity, NULL,
                           SVN_INVALID_REVNUM, cancel_func, cancel_baton,
                           notify_func, notify_baton, pool));
 
+      /* Copy properties. */
+      SVN_ERR(copy_props(src_path, dst_path,
+                         src_access, dst_parent_access,
+                         pool));
+
       /* Get the accesses for the newly added dir and its source, we'll
          need both to process any of SRC_PATHS's children below. */
       SVN_ERR(svn_wc_adm_retrieve(&dst_child_dir_access, dst_parent_access,
@@ -223,11 +267,15 @@ copy_added_dir_administratively(const char *src_path,
           SVN_ERR(svn_wc_entry(&entry, src_fullpath, src_child_dir_access,
                                TRUE, subpool));
 
+          /* We do not need to handle excluded items here, since this function
+             only deal with the sources which are not yet in the repos.
+             Exclude flag is by definition not expected in such situation. */
+
           /* Recurse on directories; add files; ignore the rest. */
           if (this_entry.filetype == APR_DIR)
             {
               SVN_ERR(copy_added_dir_administratively(src_fullpath,
-                                                      entry ? TRUE : FALSE,
+                                                      entry != NULL,
                                                       dst_child_dir_access,
                                                       src_child_dir_access,
                                                       this_entry.name,
@@ -240,7 +288,8 @@ copy_added_dir_administratively(const char *src_path,
           else if (this_entry.filetype != APR_UNKFILE)
             {
               SVN_ERR(copy_added_file_administratively(src_fullpath,
-                                                       entry ? TRUE : FALSE,
+                                                       entry != NULL,
+                                                       src_child_dir_access,
                                                        dst_child_dir_access,
                                                        this_entry.name,
                                                        cancel_func,
@@ -272,8 +321,15 @@ get_copyfrom_url_rev_via_parent(const char *src_path,
                                 svn_wc_adm_access_t *src_access,
                                 apr_pool_t *pool)
 {
-  const char *parent_path = svn_path_dirname(src_path, pool);
-  const char *rest = svn_path_basename(src_path, pool);
+  const char *parent_path;
+  const char *rest;
+  const char *abs_src_path;
+
+  SVN_ERR(svn_path_get_absolute(&abs_src_path, src_path, pool));
+
+  parent_path = svn_path_dirname(abs_src_path, pool);
+  rest = svn_path_basename(abs_src_path, pool);
+
   *copyfrom_url = NULL;
 
   while (! *copyfrom_url)
@@ -283,8 +339,8 @@ get_copyfrom_url_rev_via_parent(const char *src_path,
 
       /* Don't look for parent_path in src_access if it can't be
          there... */
-      if (svn_path_is_ancestor(svn_wc_adm_access_path(src_access),
-                               parent_path))
+      if (svn_dirent_is_ancestor(svn_wc_adm_access_path(src_access),
+                                 parent_path))
         {
           SVN_ERR(svn_wc_adm_retrieve(&parent_access, src_access,
                                       parent_path, pool));
@@ -298,7 +354,7 @@ get_copyfrom_url_rev_via_parent(const char *src_path,
                                          NULL, NULL, pool));
           SVN_ERR(svn_wc__entry_versioned(&entry, parent_path, parent_access,
                                          FALSE, pool));
-          SVN_ERR(svn_wc_adm_close(parent_access));
+          SVN_ERR(svn_wc_adm_close2(parent_access, pool));
         }
 
       if (entry->copyfrom_url)
@@ -309,9 +365,25 @@ get_copyfrom_url_rev_via_parent(const char *src_path,
         }
       else
         {
+          const char *last_parent_path = parent_path;
+
           rest = svn_path_join(svn_path_basename(parent_path, pool),
                                rest, pool);
           parent_path = svn_path_dirname(parent_path, pool);
+
+          if (strcmp(parent_path, last_parent_path) == 0)
+            {
+              /* If this happens, it probably means that parent_path is "".
+                 But there's no reason to limit ourselves to just that case;
+                 given everything else that's going on in this function, a
+                 strcmp() is pretty cheap, and the result we're trying to
+                 prevent is an infinite loop if svn_path_dirname() returns
+                 its input unchanged. */
+              return svn_error_createf
+                (SVN_ERR_WC_COPYFROM_PATH_NOT_FOUND, NULL,
+                 _("no parent with copyfrom information found above '%s'"),
+                 svn_path_local_style(src_path, pool));
+            }
         }
     }
 
@@ -381,7 +453,9 @@ copy_file_administratively(const char *src_path,
                            svn_wc_adm_access_t *src_access,
                            svn_wc_adm_access_t *dst_parent,
                            const char *dst_basename,
-                           svn_wc_notify_func2_t notify_copied,
+                           svn_cancel_func_t cancel_func,
+                           void *cancel_baton,
+                           svn_wc_notify_func2_t notify_func,
                            void *notify_baton,
                            apr_pool_t *pool)
 {
@@ -392,10 +466,6 @@ copy_file_administratively(const char *src_path,
   const char *dst_path
     = svn_path_join(svn_wc_adm_access_path(dst_parent), dst_basename, pool);
 
-  /* Discover the paths to the two text-base files */
-  const char *src_txtb = svn_wc__text_base_path(src_path, FALSE, pool);
-  const char *tmp_txtb = svn_wc__text_base_path(dst_path, TRUE, pool);
-
   /* Sanity check:  if dst file exists already, don't allow overwrite. */
   SVN_ERR(svn_io_check_path(dst_path, &dst_kind, pool));
   if (dst_kind != svn_node_none)
@@ -403,17 +473,16 @@ copy_file_administratively(const char *src_path,
                              _("'%s' already exists and is in the way"),
                              svn_path_local_style(dst_path, pool));
 
-  /* Even if DST_PATH doesn't exist it may still be a versioned file; it
+  /* Even if DST_PATH doesn't exist it may still be a versioned item; it
      may be scheduled for deletion, or the user may simply have removed the
      working copy.  Since we are going to write to DST_PATH text-base and
      prop-base we need to detect such cases and abort. */
   SVN_ERR(svn_wc_entry(&dst_entry, dst_path, dst_parent, FALSE, pool));
-  if (dst_entry && dst_entry->kind == svn_node_file)
+  if (dst_entry && dst_entry->schedule != svn_wc_schedule_delete)
     {
-      if (dst_entry->schedule != svn_wc_schedule_delete)
-        return svn_error_createf(SVN_ERR_ENTRY_EXISTS, NULL,
-                                 _("There is already a versioned item '%s'"),
-                                 svn_path_local_style(dst_path, pool));
+      return svn_error_createf(SVN_ERR_ENTRY_EXISTS, NULL,
+                               _("There is already a versioned item '%s'"),
+                               svn_path_local_style(dst_path, pool));
     }
 
   /* Sanity check 1: You cannot make a copy of something that's not
@@ -435,9 +504,10 @@ copy_file_administratively(const char *src_path,
   /* Schedule the new file for addition in its parent, WITH HISTORY. */
   {
     const char *copyfrom_url;
-    const char *tmp_wc_text;
     svn_revnum_t copyfrom_rev;
     apr_hash_t *props, *base_props;
+    svn_stream_t *base_contents;
+    svn_stream_t *contents;
 
     /* Are we moving or copying a file that is already moved or copied
        but not committed? */
@@ -462,42 +532,72 @@ copy_file_administratively(const char *src_path,
     SVN_ERR(svn_wc__load_props(&base_props, &props, NULL, src_access,
                                src_path, pool));
 
-    /* Copy pristine text-base to temporary location. */
-    SVN_ERR(svn_io_copy_file(src_txtb, tmp_txtb, TRUE, pool));
-
     /* Copy working copy file to temporary location */
     {
       svn_boolean_t special;
 
-      SVN_ERR(svn_wc_create_tmp_file2(NULL, &tmp_wc_text,
-                                      svn_wc_adm_access_path(dst_parent),
-                                      svn_io_file_del_none, pool));
-
       SVN_ERR(svn_wc__get_special(&special, src_path, src_access, pool));
       if (special)
         {
-          SVN_ERR(svn_subst_copy_and_translate3(src_path, tmp_wc_text,
-                                                NULL, FALSE, NULL,
-                                                FALSE, special, pool));
+          SVN_ERR(svn_subst_read_specialfile(&contents, src_path,
+                                             pool, pool));
         }
       else
-        SVN_ERR(svn_io_copy_file(src_path, tmp_wc_text, TRUE, pool));
+        {
+          svn_subst_eol_style_t eol_style;
+          const char *eol_str;
+          apr_hash_t *keywords;
+
+          SVN_ERR(svn_wc__get_keywords(&keywords, src_path, src_access, NULL,
+                                       pool));
+          SVN_ERR(svn_wc__get_eol_style(&eol_style, &eol_str, src_path,
+                                        src_access, pool));
+
+          SVN_ERR(svn_stream_open_readonly(&contents, src_path, pool, pool));
+
+          if (svn_subst_translation_required(eol_style, eol_str, keywords,
+                                             FALSE, FALSE))
+            {
+              svn_boolean_t repair = FALSE;
+
+              if (eol_style == svn_subst_eol_style_native)
+                eol_str = SVN_SUBST_NATIVE_EOL_STR;
+              else if (eol_style == svn_subst_eol_style_fixed)
+                repair = TRUE;
+              else if (eol_style != svn_subst_eol_style_none)
+                return svn_error_create(SVN_ERR_IO_UNKNOWN_EOL, NULL, NULL);
+
+              /* Wrap the stream to translate to normal form */
+              contents = svn_subst_stream_translated(contents,
+                                                     eol_str,
+                                                     repair,
+                                                     keywords,
+                                                     FALSE /* expand */,
+                                                     pool);
+            }
+        }
     }
 
-    SVN_ERR(svn_wc_add_repos_file2(dst_path, dst_parent,
-                                   tmp_txtb, tmp_wc_text,
+    SVN_ERR(svn_wc_get_pristine_contents(&base_contents, src_path,
+                                         pool, pool));
+
+    SVN_ERR(svn_wc_add_repos_file3(dst_path, dst_parent,
+                                   base_contents, contents,
                                    base_props, props,
-                                   copyfrom_url, copyfrom_rev, pool));
+                                   copyfrom_url, copyfrom_rev,
+                                   cancel_func, cancel_baton,
+                                   notify_func, notify_baton,
+                                   pool));
   }
 
   /* Report the addition to the caller. */
-  if (notify_copied != NULL)
+  if (notify_func != NULL)
     {
       svn_wc_notify_t *notify = svn_wc_create_notify(dst_path,
                                                      svn_wc_notify_add,
                                                      pool);
       notify->kind = svn_node_file;
-      (*notify_copied)(notify_baton, notify, pool);
+      (*notify_func)(notify_baton, notify, pool);
     }
 
   return SVN_NO_ERROR;
@@ -523,15 +623,12 @@ post_copy_cleanup(svn_wc_adm_access_t *adm_access,
   /* Remove wcprops. */
   SVN_ERR(svn_wc__props_delete(path, svn_wc__props_wcprop, adm_access, pool));
 
-  /* Read this directory's entries file. */
-  SVN_ERR(svn_wc_entries_read(&entries, adm_access, FALSE, pool));
-
   /* Because svn_io_copy_dir_recursively() doesn't copy directory
      permissions, we'll patch up our tree's .svn subdirs to be
      hidden. */
 #ifdef APR_FILE_ATTR_HIDDEN
   {
-    const char *adm_dir = svn_wc__adm_path(path, FALSE, pool, NULL);
+    const char *adm_dir = svn_wc__adm_child(path, NULL, pool);
     const char *path_apr;
     apr_status_t status;
     SVN_ERR(svn_path_cstring_from_utf8(&path_apr, adm_dir, pool));
@@ -563,6 +660,9 @@ post_copy_cleanup(svn_wc_adm_access_t *adm_access,
       kind = entry->kind;
       deleted = entry->deleted;
 
+      if (entry->depth == svn_depth_exclude)
+        continue;
+
       /* Convert deleted="true" into schedule="delete" for all
          children (and grandchildren, if RECURSE is set) of the path
          represented by ADM_ACCESS.  The result of this is that when
@@ -753,15 +853,13 @@ copy_dir_administratively(const char *src_path,
         copyfrom_url = tmp;
       }
 
-    SVN_ERR(svn_wc_adm_close(adm_access));
+    SVN_ERR(svn_wc_adm_close2(adm_access, pool));
 
-    SVN_ERR(svn_wc_add2(dst_path, dst_parent,
-                        copyfrom_url, copyfrom_rev,
-                        cancel_func, cancel_baton,
-                        notify_copied, notify_baton, pool));
+    return svn_wc_add3(dst_path, dst_parent, svn_depth_infinity,
+                       copyfrom_url, copyfrom_rev,
+                       cancel_func, cancel_baton,
+                       notify_copied, notify_baton, pool);
   }
-
-  return SVN_NO_ERROR;
 }
 
 
@@ -780,8 +878,8 @@ svn_wc_copy2(const char *src_path,
 {
   svn_wc_adm_access_t *adm_access;
   svn_node_kind_t src_kind;
-  const char *dst_path;
-  const svn_wc_entry_t *dst_entry, *src_entry;
+  const char *dst_path, *target_path;
+  const svn_wc_entry_t *dst_entry, *src_entry, *target_entry;
 
   SVN_ERR(svn_wc_adm_probe_open3(&adm_access, NULL, src_path, FALSE, -1,
                                  cancel_func, cancel_baton, pool));
@@ -806,6 +904,21 @@ svn_wc_copy2(const char *src_path,
        _("Cannot copy to '%s' as it is scheduled for deletion"),
        svn_path_local_style(svn_wc_adm_access_path(dst_parent), pool));
 
+  /* TODO(#2843): Rework the error report. */
+  /* Check if the copy target is missing or hidden and thus not exist on the
+     disk, before actually doing the file copy. */
+  target_path = svn_path_join(dst_path, dst_basename, pool);
+  SVN_ERR(svn_wc_entry(&target_entry, target_path, dst_parent, TRUE, pool));
+  if (target_entry
+      && ((target_entry->depth == svn_depth_exclude)
+          || target_entry->absent))
+    {
+      return svn_error_createf
+        (SVN_ERR_ENTRY_EXISTS,
+         NULL, _("'%s' is already under version control"),
+         svn_path_local_style(target_path, pool));
+    }
+
   SVN_ERR(svn_io_check_path(src_path, &src_kind, pool));
 
   if (src_kind == svn_node_file)
@@ -815,7 +928,7 @@ svn_wc_copy2(const char *src_path,
       if (src_entry->schedule == svn_wc_schedule_add
           && (! src_entry->copied))
         {
-          SVN_ERR(copy_added_file_administratively(src_path, TRUE,
+          SVN_ERR(copy_added_file_administratively(src_path, TRUE, adm_access,
                                                    dst_parent, dst_basename,
                                                    cancel_func, cancel_baton,
                                                    notify_func, notify_baton,
@@ -825,7 +938,9 @@ svn_wc_copy2(const char *src_path,
         {
           SVN_ERR(copy_file_administratively(src_path, adm_access,
                                              dst_parent, dst_basename,
-                                             notify_func, notify_baton, pool));
+                                             cancel_func, cancel_baton,
+                                             notify_func, notify_baton,
+                                             pool));
         }
     }
   else if (src_kind == svn_node_dir)
@@ -837,9 +952,10 @@ svn_wc_copy2(const char *src_path,
         {
           SVN_ERR(copy_added_dir_administratively(src_path, TRUE,
                                                   dst_parent, adm_access,
-                                                  dst_basename, cancel_func,
-                                                  cancel_baton, notify_func,
-                                                  notify_baton, pool));
+                                                  dst_basename,
+                                                  cancel_func, cancel_baton,
+                                                  notify_func, notify_baton,
+                                                  pool));
         }
       else
         {
@@ -850,10 +966,7 @@ svn_wc_copy2(const char *src_path,
         }
     }
 
-  SVN_ERR(svn_wc_adm_close(adm_access));
-
-
-  return SVN_NO_ERROR;
+  return svn_wc_adm_close2(adm_access, pool);
 }
 
 
