@@ -1,7 +1,7 @@
 /* changes-test.c --- test `changes' interfaces
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004, 2008 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -22,12 +22,13 @@
 
 #include <apr.h>
 
+#include "../svn_test.h"
+
 #include "svn_pools.h"
 #include "svn_error.h"
+#include "private/svn_skel.h"
 
-#include "../svn_test.h"
 #include "../svn_test_fs.h"
-#include "../../libsvn_fs_base/util/skel.h"
 #include "../../libsvn_fs_base/util/fs_skels.h"
 #include "../../libsvn_fs_base/bdb/changes-table.h"
 
@@ -123,7 +124,7 @@ add_standard_changes(svn_fs_t *fs,
 
       /* Write new changes to the changes table. */
       SVN_ERR(svn_fs_base__retry_txn(args.fs, txn_body_changes_add, &args,
-                                     pool));
+                                     TRUE, pool));
     }
 
   return SVN_NO_ERROR;
@@ -173,8 +174,8 @@ changes_add(const char **msg,
     return SVN_NO_ERROR;
 
   /* Create a new fs and repos */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-changes-add",
-                              "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs(&fs, "test-repo-changes-add", opts,
+                                  pool));
 
   /* Add the standard slew of changes. */
   SVN_ERR(add_standard_changes(fs, pool));
@@ -201,15 +202,15 @@ changes_fetch_raw(const char **msg,
     return SVN_NO_ERROR;
 
   /* Create a new fs and repos */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-changes-fetch",
-                              "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs(&fs, "test-repo-changes-fetch", opts,
+                                  pool));
 
   /* First, verify that we can request changes for an arbitrary key
      without error. */
   args.fs = fs;
   args.key = "blahbliggityblah";
-  SVN_ERR(svn_fs_base__retry_txn(args.fs, txn_body_changes_fetch_raw,
-                                 &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(args.fs, txn_body_changes_fetch_raw, &args,
+                                 FALSE, pool));
   if ((! args.raw_changes) || (args.raw_changes->nelts))
     return svn_error_create(SVN_ERR_TEST_FAILED, NULL,
                             "expected empty changes array");
@@ -232,7 +233,7 @@ changes_fetch_raw(const char **msg,
 
       /* And get those changes. */
       SVN_ERR(svn_fs_base__retry_txn(args.fs, txn_body_changes_fetch_raw,
-                                     &args, pool));
+                                     &args, FALSE, pool));
       if (! args.raw_changes)
         return svn_error_createf(SVN_ERR_TEST_FAILED, NULL,
                                  "got no changes for key '%s'", txn_id);
@@ -309,8 +310,8 @@ changes_delete(const char **msg,
     return SVN_NO_ERROR;
 
   /* Create a new fs and repos */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-changes-delete",
-                              "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs(&fs, "test-repo-changes-delete", opts,
+                                  pool));
 
   /* Add the standard slew of changes. */
   SVN_ERR(add_standard_changes(fs, pool));
@@ -321,10 +322,10 @@ changes_delete(const char **msg,
       args.fs = fs;
       args.key = standard_txns[i];
       SVN_ERR(svn_fs_base__retry_txn(args.fs, txn_body_changes_delete,
-                                     &args, pool));
+                                     &args, FALSE, pool));
       args.changes = 0;
       SVN_ERR(svn_fs_base__retry_txn(args.fs, txn_body_changes_fetch_raw,
-                                     &args, pool));
+                                     &args, FALSE, pool));
       if ((! args.raw_changes) || (args.raw_changes->nelts))
         return svn_error_createf
           (SVN_ERR_TEST_FAILED, NULL,
@@ -505,14 +506,15 @@ changes_fetch(const char **msg,
     return SVN_NO_ERROR;
 
   /* Create a new fs and repos */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-changes-fetch",
-                              "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs(&fs, "test-repo-changes-fetch", opts,
+                                  pool));
 
   /* First, verify that we can request changes for an arbitrary key
      without error. */
   args.fs = fs;
   args.key = "blahbliggityblah";
-  SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_changes_fetch, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_changes_fetch, &args,
+                                 FALSE, pool));
   if ((! args.changes) || (apr_hash_count(args.changes)))
     return svn_error_create(SVN_ERR_TEST_FAILED, NULL,
                             "expected empty changes hash");
@@ -538,7 +540,7 @@ changes_fetch(const char **msg,
       /* And get those changes via in the internal interface, and
          verify that they are accurate. */
       SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_changes_fetch, &args,
-                                     pool));
+                                     FALSE, pool));
       if (! args.changes)
         return svn_error_createf
           (SVN_ERR_TEST_FAILED, NULL,
@@ -575,9 +577,9 @@ changes_fetch_ordering(const char **msg,
     return SVN_NO_ERROR;
 
   /* Create a new fs and repos */
-  SVN_ERR(svn_test__create_fs
-          (&fs, "test-repo-changes-fetch-ordering",
-           "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs
+          (&fs, "test-repo-changes-fetch-ordering", opts,
+           pool));
 
   /*** REVISION 1: Make some files and dirs. ***/
   SVN_ERR(svn_fs_begin_txn(&txn, fs, youngest_rev, subpool));
@@ -620,7 +622,7 @@ changes_fetch_ordering(const char **msg,
   args.fs = fs;
   args.key = txn_name;
   SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_changes_fetch, &args,
-                                 subpool));
+                                 FALSE, subpool));
   if ((! args.changes) || (apr_hash_count(args.changes) != 3))
     return svn_error_create(SVN_ERR_TEST_FAILED, NULL,
                             "expected changes");
@@ -692,7 +694,7 @@ changes_fetch_ordering(const char **msg,
   args.fs = fs;
   args.key = txn_name;
   SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_changes_fetch, &args,
-                                 subpool));
+                                 FALSE, subpool));
   if ((! args.changes) || (apr_hash_count(args.changes) != 3))
     return svn_error_create(SVN_ERR_TEST_FAILED, NULL,
                             "expected changes");
